#! /usr/bin/python

# Requires kodipydent module

import sys
sys.path.append('/usr/osmc/lib/python2.7/dist-packages/')

try:
    import cec
except:
    sys.path.append('/usr/osmc/lib/python2.7/dist-packages/cec')
    import cec

import argparse
import logging
import logging.handlers
import time
from kodipydent import Kodi

log_level = cec.CEC_LOG_DEBUG

class CecWatcher(object):
    cecconfig = cec.libcec_configuration()
    ceclib = {}

    def __init__(self):
        self.cecconfig.strDeviceName   = "Kodi CEC"
        self.cecconfig.bActivateSource = 0
        self.cecconfig.deviceTypes.Add(cec.CEC_DEVICE_TYPE_RECORDING_DEVICE)
        self.cecconfig.clientVersion = cec.LIBCEC_VERSION_CURRENT

    def set_callbacks(self, command_callback, key_press_callback, log_callback):
        self.cecconfig.SetLogCallback(log_callback)
        self.cecconfig.SetKeyPressCallback(key_press_callback)
        self.cecconfig.SetCommandCallback(command_callback)

    def register(self):
        self.ceclib = cec.ICECAdapter.Create(self.cecconfig)

        adapters = self.ceclib.DetectAdapters()
        adapter = None
        for adapter in adapters:
            adapter = adapter.strComName
            logger.info("Found CEC adapter: " + adapter)

        if adapter == None:
            raise EnvironmentError("No adapters found")
        else:
            if self.ceclib.Open(adapter):
                logger.info("CEC adapter opened")
            else:
                raise EnvironmentError("Failed to open a connection to the CEC adapter")

    def log_callback(self, level, time, message):
        try:
            if level <= log_level:
                if level == cec.CEC_LOG_ERROR:
                  logger.error(message)
                elif level == cec.CEC_LOG_WARNING:
                  logger.warn(message)
                elif level == cec.CEC_LOG_NOTICE:
                  logger.info(message)
                elif level == cec.CEC_LOG_TRAFFIC:
                  logger.debug(message)
                elif level == cec.CEC_LOG_DEBUG:
                  logger.debug(message)
        except:
            logger.exception("Unable to log")
        return 0

    def key_press_callback(self, key, duration):
        return 0

    def command_callback(self, cmd):
        try:
            logger.info(u"Command: " + str(cmd))
            if cmd == ">> 0f:36" or cmd == ">> 0f:82:00:00":
                if cmd == ">> 0f:36":
                    logger.info("TV is standby, sending Kodi pause/stop")
                else:
                    logger.info("Source switched, sending Kodi pause")

                try:
                    my_kodi = Kodi("127.0.0.1", port=80)
                    logger.info("Connected to Kodi")
                    did_action = False
                    for player in my_kodi.Player.GetActivePlayers()["result"]:
                        playerid = player["playerid"]
                        logger.debug("Player: " + str(playerid))
                        try:
                            item = my_kodi.Player.GetItem(playerid=playerid)["result"]["item"];
                            logger.debug("Type: " + item["type"])
                            if item["type"] == "channel":
                                # TV - check if we stop
                                if cmd == ">> 0f:36":
                                    # TV shuts down - stop
                                    my_kodi.Player.Stop(playerid=playerid)
                                    logger.info("Stopped playback")
                                    did_action = True
                            else:
                                # Other types - just pause
                                my_kodi.Player.PlayPause(playerid=playerid, play=False)
                                logger.info("Paused playback")
                                did_action = True
                        except:
                            logger.exception("Unable to stop player")
                    if not did_action:
                        logger.info("Player not running any channel, nothing to do")
                except:
                    logger.exception("Unable to contact Kodi")
        except:
            logger.exception("Error handling command callback")

        return 0

# logging callback
def log_callback(level, time, message):
  return watcher.log_callback(level, time, message)

# key press callback
def key_press_callback(key, duration):
  return watcher.key_press_callback(key, duration)

# command callback
def command_callback(cmd):
  return watcher.command_callback(cmd)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(prog=sys.argv[0], description="Watch CEC and stop TV playback when TV goes standby")
    parser.add_argument("-c", "--console", dest="console", action="store_true", default=False, help="log to console instead of syslog")

    args = parser.parse_args()

    keep_files = []
    logger = logging.getLogger('CecWatch')
    logger.setLevel(logging.DEBUG)

    if args.console:
        # Logging to console (unbuffered output)
        import os
        stdout_dup = os.fdopen(os.dup(sys.stdout.fileno()), 'w', 0)
        handler = logging.StreamHandler(stream = stdout_dup)
        keep_files += [stdout_dup]
    else:
        # Logging to syslog
        handler = logging.handlers.SysLogHandler(address = '/dev/log')
        keep_files += [handler.socket]

    logger.addHandler(handler)

    # initialise libCEC
    try:
        logger.info("Registering to CEC")

        watcher = CecWatcher()
        watcher.set_callbacks(command_callback, key_press_callback, log_callback)
        watcher.register()

        try:
            while True:
                time.sleep(1)
        except SystemExit:
            pass
    except:
        logger.exception("Failure running CEC watcher")
